/*
 *  Sample C.    Lexical analysis
 */

%{
  #include "symbol.c"
  #define yywrap() 1

  int binary();
  void yycomment( char * mat) ;
%}

letter                   [a-zA-Z_]
digit                    [0-9]
letter_or_digit          [a-zA-Z_0-9]
white_space              [ \t\n]
other                    .
%option yylineno
%%

{white_space}+              ;

"/*"                        yycomment("*/");

{letter}{letter_or_digit}*  return binary();

{digit}+                    { yylval.value = strtol( yytext, (char**)0, 10);
                              if ( (yylval.value < 32767) &&
                                   (yylval.value >= 0) )
                                return CONST;
                              error("integer Overflow");
                            }

">="                        return GE;
"<="                        return LE;
"=="                        return EQ;
"!="                        return NE;
"+="                        return PE;
"-="                        return ME;
"*="                        return TE;
"/="                        return DE;
"%="                        return RE;
"++"                        return PP;
"--"                        return MM;

{other}                     return yytext[0];

%%

/*
 *  reserved word screener
 */


struct rwtable        /* reserved word table */
{
  char * name;               /* representation */
  int  token;                /* yylex() value  */
} reservedWords[] =          /* sorted */
  {
    "break" , BREAK,
    "continue", CONTINUE,
    "else", ELSE,
    "if", IF,
    "int", INT,
    "return", RETURN,
    "while", WHILE,
  };

int binary()
{
  struct rwtable * low = reservedWords,
                 * high = ( reservedWords - 1 + (sizeof(reservedWords)/sizeof(reservedWords[0]))),
                 * mid = NULL;
  int c;

  while ( low <= high )
  {
    mid = low + ( high - low) / 2;
    if ( ( c = strcmp(mid->name, yytext) ) == 0 )
      return mid->token;
    else if ( c < 0 )
      low = mid + 1;
    else
      high = mid - 1;
  } /* while */

  yylval.ptr = lookUp( yytext );
  return ID;

}/* binary */

void yyerror(char *s){
  printf("%s\n", s);
} /* yyerror */

void yycomment( char * mat) {
  int c;
  char *cp;

  for (cp = mat; *cp != 0; ) {
    c = input();
    if (c == EOF) {
      yyerror("end of file in comment");
      break;
    }
    if (c != *cp++) {
      cp = mat;
      if (c == *cp)
      	cp++;
    }
  } /* for */
} /* yycomment */
